require 'sylvilagus/init'
require 'sylvilagus/ch03'

require 'rabbitmq-client.jar'
import com.rabbitmq.client.ConnectionFactory
import com.rabbitmq.client.DefaultConsumer
import org.jruby.RubyString

class Sylvilagus::Ch03::LogListeners
  class LogConsumer < DefaultConsumer
    attr_accessor :level

    def handleDelivery(consumer_tag, envelope, properties, body)
      body_string = RubyString.bytes_to_string(body)
      puts "#{level}: #{body_string}"
      channel.basic_ack(envelope.delivery_tag, false)
    end
  end

  def main
    factory = ConnectionFactory.new
    amqp_uri = ENV.fetch('SYLVILAGUS_ROOT_ADMIN_AMQP_URI')
    factory.uri = amqp_uri
    puts "Getting connection for #{amqp_uri.inspect}..."
    @conn = factory.new_connection

    trap :INT do
      begin
        @conn.close
        exit 1
      rescue NativeException
        exit 2
      end
    end

    puts 'Getting channel...'
    channel = @conn.create_channel

    puts 'Declaring queues...'
    errors_queue = channel.queue_declare.get_queue
    warnings_queue = channel.queue_declare.get_queue
    info_queue = channel.queue_declare.get_queue

    puts "Binding queues to 'amq.rabbitmq.log' exchange..."
    channel.queue_bind(errors_queue, 'amq.rabbitmq.log', 'error')
    channel.queue_bind(warnings_queue, 'amq.rabbitmq.log', 'warning')
    channel.queue_bind(info_queue, 'amq.rabbitmq.log', 'info')

    errors_consumer = LogConsumer.new(channel)
    errors_consumer.level = 'error'
    warnings_consumer = LogConsumer.new(channel)
    warnings_consumer.level = 'warning'
    info_consumer = LogConsumer.new(channel)
    info_consumer.level = 'info'

    puts 'Setting up consumers...'
    channel.basic_consume(errors_queue, false, errors_consumer)
    channel.basic_consume(warnings_queue, false, warnings_consumer)
    channel.basic_consume(info_queue, false, info_consumer)

    loop do
      sleep 1
    end
  ensure
    begin
      @conn.close if @conn
    rescue NativeException
    end
  end
end

if $0 == __FILE__
  Sylvilagus::Ch03::LogListeners.new.main
end
