from __future__ import print_function

import json
import sys

import pika

from argparse import ArgumentParser


def main(sysargs=sys.argv[:]):
    parser = ArgumentParser()
    parser.add_argument('-r',
                        '--routing-key',
                        help='Routing key for message (e.g. myalert.im)')
    parser.add_argument('-m',
                        '--message',
                        help='Message text for alert.')

    args = parser.parse_args(sysargs[1:])

    creds_broker = pika.PlainCredentials('alert_user', 'alertme')
    conn_params = pika.ConnectionParameters('localhost',
                                            virtual_host='/',
                                            credentials=creds_broker)
    conn_broker = pika.BlockingConnection(conn_params)

    channel = conn_broker.channel()

    msg = json.dumps({'message': args.message})
    msg_props = pika.BasicProperties()
    msg_props.content_type = 'application/json'
    msg_props.durable = False

    channel.basic_publish(body=msg,
                          exchange='alerts',
                          properties=msg_props,
                          routing_key=args.routing_key)

    print(
        ('Sent message {} tagged with routing key {!r} to ' +
         'exchange "alerts" on vhost "/".').format(msg, args.routing_key)
    )

    conn_broker.close()

    return 0


if __name__ == '__main__':
    sys.exit(main())
